@@ -26,14 +26,14 @@ static void map_way_color(potlatch_conf_t *potlatch, way_t *way) {
   potlatch_color_t *plcol = potlatch->colors;
 
   /* a highway/waterway/railway is never a polygon */
-  if(!gpx_way_is_way(way)) {
+  if(!osm_way_is_way(way)) {
   
     /* check if this way is a polygon */
     node_chain_t *node_chain = way->node_chain;
     while(node_chain && node_chain->next) node_chain = node_chain->next;
     if(way->node_chain && node_chain && 
        way->node_chain->node == node_chain->node) {
-      way->draw_flags |= GPX_DRAW_FLAGS_POLYGON;
+      way->draw_flags |= OSM_DRAW_FLAGS_POLYGON;
       
       /* default fill color is light grey */
       way->fill_color = (0xc0c0c0 << 8) | FILL_TRANSP;
@@ -41,9 +41,9 @@ static void map_way_color(potlatch_conf_t *potlatch, way_t *way) {
   }
   
   while(plcol) {  // (way->color == MAP_COLOR_NONE && plcol) 
-    if(gpx_way_has_key_or_value(way, plcol->tag)) {
+    if(osm_way_has_key_or_value(way, plcol->tag)) {
       way->color = (plcol->color << 8) | 0xff;
-      if(plcol->stroke) way->draw_flags |= GPX_DRAW_FLAGS_STROKE;
+      if(plcol->stroke) way->draw_flags |= OSM_DRAW_FLAGS_STROKE;
       
       if(plcol->fill_color != COLOR_NONE)
 	way->fill_color = (plcol->fill_color << 8) | FILL_TRANSP;
@@ -55,8 +55,8 @@ static void map_way_color(potlatch_conf_t *potlatch, way_t *way) {
   if(way->color == MAP_COLOR_NONE) way->color = DARK_GREY;
 }
 
-static void map_init_colors(potlatch_conf_t *potlatch, gpx_t *gpx) {
-  way_t *way = gpx->way;
+static void map_init_colors(potlatch_conf_t *potlatch, osm_t *osm) {
+  way_t *way = osm->way;
 
   printf("preparing colors\n");
 
@@ -137,29 +137,15 @@ static void map_node_select(appdata_t *appdata, node_t *node) {
   map_item_t *new_map_item = g_new0(map_item_t, 1);
   memcpy(new_map_item, map_item, sizeof(map_item_t));
   new_map_item->highlight = TRUE;
-
-  map_hl_new(map, GNOME_CANVAS_GROUP(map->group[MAP_GROUP_NODES_HL]),
-	     GNOME_TYPE_CANVAS_ELLIPSE, new_map_item,
-	     "x1", (double)x-2*RADIUS, "x2", (double)x+2*RADIUS,
-	     "y1", (double)y-2*RADIUS, "y2", (double)y+2*RADIUS,
-	     "fill_color_rgba", HIGHLIGHT_COLOR, 
-	     "width-units", 0.0,
-	     NULL);       
+  map_hl_circle_new(map, CANVAS_GROUP_NODES_HL, new_map_item, 
+		    x, y, 2*RADIUS, HIGHLIGHT_COLOR);
 
   if(!map_item->item) {
     /* and draw a fake node */
     new_map_item = g_new0(map_item_t, 1);
     memcpy(new_map_item, map_item, sizeof(map_item_t));
     new_map_item->highlight = TRUE;
-
-    map_hl_new(map, 
-	       GNOME_CANVAS_GROUP(map->group[MAP_GROUP_NODES_HL]),
-	       GNOME_TYPE_CANVAS_ELLIPSE, new_map_item,
-	       "x1", (double)x-RADIUS, "x2", (double)x+RADIUS,
-	       "y1", (double)y-RADIUS, "y2", (double)y+RADIUS,
-	       "fill_color_rgba", HIGHLIGHT2_COLOR, 
-	       "width-units", 0.0,
-	       NULL);       
+    map_hl_circle_new(map, CANVAS_GROUP_NODES_HL, new_map_item, x, y, RADIUS, HIGHLIGHT2_COLOR);
   }
 }
 
@@ -193,32 +179,24 @@ static void map_way_select(appdata_t *appdata, way_t *way) {
     
       gint x = node_chain->node->lpos.x;
       gint y = node_chain->node->lpos.y;
-      map_hl_new(map, 
-		 GNOME_CANVAS_GROUP(map->group[MAP_GROUP_NODES_HL]),
-		 GNOME_TYPE_CANVAS_ELLIPSE, new_map_item,
-		 "x1", (double)x-RADIUS, "x2", (double)x+RADIUS,
-		 "y1", (double)y-RADIUS, "y2", (double)y+RADIUS,
-		 "fill_color_rgba", HIGHLIGHT2_COLOR, 
-		 "width-units", 0.0,
-		 NULL);       
+
+      map_hl_circle_new(map, CANVAS_GROUP_NODES_HL, new_map_item, x, y, RADIUS, HIGHLIGHT2_COLOR);
     }
 
     node_chain = node_chain->next;
   }
 
   /* a way needs at least 2 points to be drawn */
-  guint nodes = gpx_way_number_of_nodes(way);
+  guint nodes = osm_way_number_of_nodes(way);
   if(nodes > 1) {
     
     /* allocate space for nodes */
-    GnomeCanvasPoints *points = gnome_canvas_points_new(nodes);
+    canvas_points_t *points = canvas_points_new(nodes);
 
     int node = 0;
     node_chain = map_item->way->node_chain;
     while(node_chain) {
-      points->coords[node++] = node_chain->node->lpos.x;
-      points->coords[node++] = node_chain->node->lpos.y;
-      
+      canvas_point_set_pos(points, node++, &node_chain->node->lpos);
       node_chain = node_chain->next;
     }
     
@@ -227,17 +205,11 @@ static void map_way_select(appdata_t *appdata, way_t *way) {
     memcpy(new_map_item, map_item, sizeof(map_item_t));
     new_map_item->highlight = TRUE;
     
-    map_hl_new(map, GNOME_CANVAS_GROUP(map->group[MAP_GROUP_WAYS_HL]),
-	       GNOME_TYPE_CANVAS_LINE, new_map_item,
-	       "points", points,
-	       "fill_color_rgba", HIGHLIGHT_COLOR,
-	       "width-units", (map_item->way->draw_flags & GPX_DRAW_FLAGS_STROKE)?
-	       2*STROKE_WIDTH:3*WAY_WIDTH,
-	       "join-style", GDK_JOIN_ROUND,
-	       "cap-style", GDK_CAP_ROUND,
-	       NULL);   
-  
-    gnome_canvas_points_free(points);
+    map_hl_polyline_new(map, CANVAS_GROUP_WAYS_HL, new_map_item, points, 
+			(map_item->way->draw_flags & OSM_DRAW_FLAGS_STROKE)?
+	                 2*STROKE_WIDTH:3*WAY_WIDTH, HIGHLIGHT_COLOR);
+
+    canvas_points_free(points);
   }
 }
 
@@ -275,7 +247,7 @@ static void map_item_deselect(appdata_t *appdata) {
 static gint map_item_destroy_event(GtkWidget *widget, gpointer data) {
   map_item_t *map_item = (map_item_t*)data;
 
-  //   printf("destroying map_item @ %p\n", map_item);
+  //  printf("destroying map_item @ %p\n", map_item);
 
   /* remove item from nodes/ways map_item_chain */
   map_item_chain_t **chain = NULL;
@@ -304,154 +276,101 @@ static gint map_item_destroy_event(GtkWidget *widget, gpointer data) {
   return FALSE;
 }
 
-static GnomeCanvasItem *map_item_new(GnomeCanvasGroup *parent,
-				     GType type, map_item_t *map_item,
-				     const gchar *first_arg_name,
-				     va_list args) {
-
-  map_item->item = g_object_new(type, NULL);
-  gnome_canvas_item_construct(map_item->item, parent, first_arg_name, args);
-  gtk_object_set_user_data(GTK_OBJECT(map_item->item), map_item);
-
-  gtk_signal_connect(GTK_OBJECT(map_item->item), 
-     "destroy", G_CALLBACK(map_item_destroy_event), map_item);
-  
-  return map_item->item;
-};
-
-static GnomeCanvasItem *map_node_new(GnomeCanvasGroup *parent,
-				     GType type, node_t *node,
-				     const gchar *first_arg_name,
-				     ...) {
-  va_list args;
-  va_start( args, first_arg_name);
+static canvas_item_t *map_node_new(map_t *map, node_t *node, gint radius, 
+				   gint width, canvas_color_t fill, canvas_color_t border) {
 
   map_item_t *map_item = g_new0(map_item_t, 1);
   map_item->type = MAP_TYPE_NODE;
   map_item->node = node;
-
+  map_item->item = canvas_circle_new(map, CANVAS_GROUP_NODES, 
+		     node->lpos.x, node->lpos.y, radius, width, fill, border);
+  
   /* attach map_item to nodes map_item_chain */
   map_item_chain_t **chain = &node->map_item_chain;
   while(*chain) chain = &(*chain)->next;
   *chain = g_new0(map_item_chain_t, 1);
   (*chain)->map_item = map_item;
 
-  return map_item_new(parent, type, map_item, first_arg_name, args);
+  canvas_item_set_user_data(map_item->item, map_item);
+
+  canvas_item_destroy_connect(map_item->item, 
+          G_CALLBACK(map_item_destroy_event), map_item);
+
+  return map_item->item;
 }
 
-static GnomeCanvasItem *map_way_new(GnomeCanvasGroup *parent,
-				    GType type, way_t *way,
-				    const gchar *first_arg_name,
-				    ...) {
-  va_list args;
-  va_start( args, first_arg_name);
-
+static canvas_item_t *map_way_new(map_t *map, canvas_group_t group, way_t *way, 
+				  canvas_points_t *points, gint width, canvas_color_t color,
+				  canvas_color_t fill_color) {
   map_item_t *map_item = g_new0(map_item_t, 1);
   map_item->type = MAP_TYPE_WAY;
   map_item->way = way;
 
+  if(way->draw_flags & OSM_DRAW_FLAGS_POLYGON) {
+    map_item->item = canvas_polygon_new(map, group, points, width, color, fill_color);
+  } else {
+    map_item->item = canvas_polyline_new(map, group, points, width, color);
+  }
+
   /* attach map_item to ways map_item_chain */
   map_item_chain_t **chain = &way->map_item_chain;
   while(*chain) chain = &(*chain)->next;
   *chain = g_new0(map_item_chain_t, 1);
   (*chain)->map_item = map_item;
 
-  return map_item_new(parent, type, map_item, first_arg_name, args);
+  canvas_item_set_user_data(map_item->item, map_item);
+
+  canvas_item_destroy_connect(map_item->item, 
+	      G_CALLBACK(map_item_destroy_event), map_item);
+
+  return map_item->item;
 }
 
 void map_show_node(map_t *map, node_t *node) {
-  glong x = node->lpos.x, y = node->lpos.y;
-
-  map_node_new(GNOME_CANVAS_GROUP(map->group[MAP_GROUP_NODES]),
-	       GNOME_TYPE_CANVAS_ELLIPSE, node,
-	       "x1", (double)x-S_RADIUS, "x2", (double)x+S_RADIUS,
-	       "y1", (double)y-S_RADIUS, "y2", (double)y+S_RADIUS,
-	       "fill_color_rgba", 0x000000ff,
-	       "width-units", 0.0,
-	       NULL);   
+  map_node_new(map, node, S_RADIUS, 0, NODE_WAY_COLOR, 0);
 }
 
 static void map_way_draw(map_t *map, way_t *way) {
 
   /* don't draw a way that's not there anymore */
-  if(way->flags & GPX_FLAG_DELETED)
+  if(way->flags & OSM_FLAG_DELETED)
     return;
 
   /* allocate space for nodes */
   /* a way needs at least 2 points to be drawn */
-  guint nodes = gpx_way_number_of_nodes(way);
+  guint nodes = osm_way_number_of_nodes(way);
   if(nodes > 1) {
-    GnomeCanvasPoints *points = gnome_canvas_points_new(nodes);
+    canvas_points_t *points = canvas_points_new(nodes);
     
     int node = 0;
     node_chain_t *node_chain = way->node_chain;
     while(node_chain) {
-      /* mirror map vertically within bounds */
-      points->coords[node++] = node_chain->node->lpos.x;
-      points->coords[node++] = node_chain->node->lpos.y;
+      canvas_point_set_pos(points, node++, &node_chain->node->lpos);
       node_chain = node_chain->next;
     }
     
     /* draw way */
-    if(way->draw_flags & GPX_DRAW_FLAGS_POLYGON) {
-      map_way_new(GNOME_CANVAS_GROUP(map->group[MAP_GROUP_POLYGONS]),
-		  GNOME_TYPE_CANVAS_POLYGON, way,
-		  "points", points,
-		  "fill_color_rgba", way->fill_color,
-		  "width-units", 2.0,
-		  "outline_color_rgba", DARK_GREY,
-		  "join-style", GDK_JOIN_ROUND,
-		  "cap-style", GDK_CAP_ROUND,
-		  NULL);   
+    if(way->draw_flags & OSM_DRAW_FLAGS_POLYGON) {
+      map_way_new(map, CANVAS_GROUP_POLYGONS, way, points, 2, DARK_GREY, way->fill_color);
     } else {
-      map_way_new(GNOME_CANVAS_GROUP(map->group[MAP_GROUP_WAYS]),
-		  GNOME_TYPE_CANVAS_LINE, way,
-		  "points", points,
-		  "fill_color_rgba", way->color,
-		  "width-units", WAY_WIDTH,
-		  "join-style", GDK_JOIN_ROUND,
-		  "cap-style", GDK_CAP_ROUND,
-		  NULL);   
+      map_way_new(map, CANVAS_GROUP_WAYS, way, points, WAY_WIDTH, way->color, NO_COLOR);
       
-      if(way->draw_flags & GPX_DRAW_FLAGS_STROKE)
-	map_way_new(GNOME_CANVAS_GROUP(map->group[MAP_GROUP_WAYS_OL]),
-		    GNOME_TYPE_CANVAS_LINE, way,
-		    "points", points,
-		    "fill_color_rgba", DARK_GREY,
-		    "width-units", STROKE_WIDTH,
-		    "join-style", GDK_JOIN_ROUND,
-		    "cap-style", GDK_CAP_ROUND,
-		    NULL);   
+      if(way->draw_flags & OSM_DRAW_FLAGS_STROKE)
+	map_way_new(map, CANVAS_GROUP_WAYS_OL, way, points, STROKE_WIDTH, DARK_GREY, NO_COLOR);
     }
-    gnome_canvas_points_free(points);
+    canvas_points_free(points);
   }
 }
 
 static void map_node_draw(map_t *map, node_t *node) {
-  glong x = node->lpos.x, y = node->lpos.y;
-
   /* don't draw a node that's not there anymore */
-  if(node->flags & GPX_FLAG_DELETED)
+  if(node->flags & OSM_FLAG_DELETED)
     return;
   
-  if(!node->ways) {
-    map_node_new(GNOME_CANVAS_GROUP(map->group[MAP_GROUP_NODES]),
-		 GNOME_TYPE_CANVAS_ELLIPSE, node,
-		 "x1", (double)x-RADIUS, "x2", (double)x+RADIUS,
-		 "y1", (double)y-RADIUS, "y2", (double)y+RADIUS,
-		 "fill_color_rgba", 0x008800ff,
-		 "width-units", RADIUS/2.0,
-		 "outline-color-rgba", 0x000000ff,
-		 NULL);   
-  } else if(gpx_node_has_tag(node)) {
-    map_node_new(GNOME_CANVAS_GROUP(map->group[MAP_GROUP_NODES]),
-		 GNOME_TYPE_CANVAS_ELLIPSE, node,
-		 "x1", (double)x-S_RADIUS, "x2", (double)x+S_RADIUS,
-		 "y1", (double)y-S_RADIUS, "y2", (double)y+S_RADIUS,
-		 "fill_color_rgba", 0x000000ff,
-		 "width-units", 0.0,
-		 NULL);   
-  }
+  if(!node->ways) 
+    map_node_new(map, node, RADIUS, RADIUS/2.0, NODE_COLOR, NODE_WAY_COLOR);
+  else if(osm_node_has_tag(node)) 
+    map_node_new(map, node, S_RADIUS, 0, NODE_WAY_COLOR, 0);
 }
 
 static void map_item_draw(map_t *map, map_item_t *map_item) {
@@ -485,7 +404,7 @@ static void map_item_remove(map_t *map, map_item_t *map_item) {
 
   while(chain) {
     map_item_chain_t *next = chain->next;
-    gtk_object_destroy(GTK_OBJECT(chain->map_item->item));
+    canvas_item_destroy(chain->map_item->item);
     chain = next;
   }
 }
@@ -514,18 +433,18 @@ void map_item_redraw(appdata_t *appdata, map_item_t *map_item) {
     map_item_select(appdata, &item);
 }
 
-static void map_draw(map_t *map, gpx_t *gpx) {
+static void map_draw(map_t *map, osm_t *osm) {
   g_assert(map->canvas);
 
   printf("drawing ways ...\n");
-  way_t *way = gpx->way;
+  way_t *way = osm->way;
   while(way) {
     map_way_draw(map, way);
     way = way->next;
   }
 
   printf("drawing single nodes ...\n");
-  node_t *node = gpx->node;
+  node_t *node = osm->node;
   while(node) {
     map_node_draw(map, node);
     node = node->next;
@@ -546,13 +465,13 @@ static gint map_destroy_event(GtkWidget *widget, gpointer data) {
 
 /* get the item at position x, y */
 static map_item_t *map_item_at(map_t *map, gint x, gint y) {	  
-  double wx, wy;
-  gnome_canvas_window_to_world(GNOME_CANVAS(map->canvas), x, y, &wx, &wy);
+  printf("map check at %d/%d\n", x, y);
 
-  printf("world check at %f/%f\n", wx, wy);
+  canvas_window2world(map->canvas, x, y, &x, &y);
+
+  printf("world check at %d/%d\n", x, y);
   
-  GnomeCanvasItem *item = 
-    gnome_canvas_get_item_at(GNOME_CANVAS(map->canvas), wx, wy);
+  canvas_item_t *item = canvas_get_item_at(map->canvas, x, y);
 	  
   if(!item) {
     printf("  there's no item\n");
@@ -561,8 +480,7 @@ static map_item_t *map_item_at(map_t *map, gint x, gint y) {
 
   printf("  there's an item\n");
 
-  map_item_t *map_item = 
-    (map_item_t*)gtk_object_get_user_data(GTK_OBJECT(item));
+  map_item_t *map_item = canvas_item_get_user_data(item);
 
   if(!map_item) {
     printf("  item has no user data!\n");
@@ -625,6 +543,10 @@ static map_item_t *map_real_item_at(map_t *map, gint x, gint y) {
   return map_item;
 }
 
+static void map_set_zoom(map_t *map, double zoom) {
+  map->zoom = zoom;
+  canvas_set_zoom(map->canvas, map->zoom);
+}
 
 static gboolean map_scroll_event(GtkWidget *widget, GdkEventScroll *event,
 				 gpointer data) {
@@ -632,12 +554,9 @@ static gboolean map_scroll_event(GtkWidget *widget, GdkEventScroll *event,
 
   if(event->type == GDK_SCROLL) {
     if(event->direction) 
-      appdata->map->zoom /= 1.2;
+      map_set_zoom(appdata->map, appdata->map->zoom / 1.2);
     else
-      appdata->map->zoom *= 1.2;
-
-    gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(appdata->map->canvas), 
-				     appdata->map->zoom);
+      map_set_zoom(appdata->map, appdata->map->zoom * 1.2);
   }
 
   return TRUE;
@@ -646,21 +565,28 @@ static gboolean map_scroll_event(GtkWidget *widget, GdkEventScroll *event,
 static gboolean distance_above(map_t *map, gint x, gint y, gint limit) {
   gint sx, sy;
 
+#ifdef USE_GNOMECANVAS
   gnome_canvas_get_scroll_offsets(GNOME_CANVAS(map->canvas), &sx, &sy);
 
   /* add offsets generated by mouse within map and map scrolling */
   sx = (x-map->pen_down.at.x) + (map->pen_down.so.x-sx);
   sy = (y-map->pen_down.at.y) + (map->pen_down.so.y-sy);
+#else
+  /* add offsets generated by mouse within map and map scrolling */
+  sx = (x-map->pen_down.at.x);
+  sy = (y-map->pen_down.at.y);
+#endif
 
   return(sx*sx + sy*sy > limit*limit);
 }
 
 static void map_do_scroll(map_t *map, gint x, gint y) {
   gint sx, sy;
-  gnome_canvas_get_scroll_offsets(GNOME_CANVAS(map->canvas), &sx, &sy);
-  sx -= (x-map->pen_down.at.x);
-  sy -= (y-map->pen_down.at.y);
-  gnome_canvas_scroll_to(GNOME_CANVAS(map->canvas), sx, sy);
+
+  canvas_get_scroll_offsets(map->canvas, &sx, &sy);
+  sx -= x-map->pen_down.at.x;
+  sy -= y-map->pen_down.at.y;
+  canvas_scroll_to(map->canvas, sx, sy);
 }
 
 static gboolean item_is_selected_node(map_t *map, map_item_t *map_item) {
@@ -724,7 +650,7 @@ static void map_node_move(appdata_t *appdata, map_item_t *map_item,
 			  gint ex, gint ey) {
 
   map_t *map = appdata->map;
-  gpx_t *gpx = appdata->gpx;
+  osm_t *osm = appdata->osm;
 
   g_assert(map_item->type == MAP_TYPE_NODE);
   node_t *node = map_item->node;
@@ -734,74 +660,58 @@ static void map_node_move(appdata_t *appdata, map_item_t *map_item,
 	 node->lpos.x, node->lpos.y,
 	 node->pos.lat, node->pos.lon);
 	    
-  struct { double x, y;} pos;	  
-	    
   /* convert mouse position to canvas (world) position */
-  gnome_canvas_window_to_world(GNOME_CANVAS(map->canvas), 
-			       ex, ey, &pos.x, &pos.y);
-	    
-  /* convert to integer */
-  node->lpos.x = pos.x; node->lpos.y = pos.y;
+  canvas_window2world(map->canvas, ex, ey, &node->lpos.x, &node->lpos.y);
 	    
   /* convert screen position back to utm/ll */
-  utm_t utm = gpx->bounds->center;  /* set zone/band */
-  utm.easting  =  node->lpos.x + gpx->bounds->center.easting;
-  utm.northing = -node->lpos.y + gpx->bounds->center.northing;
+  utm_t utm = osm->bounds->center;  /* set zone/band */
+  utm.easting  =  node->lpos.x + osm->bounds->center.easting;
+  utm.northing = -node->lpos.y + osm->bounds->center.northing;
   
   utm_utm2pos(&utm, &node->pos);
   printf("  now at %d %d (%f %f)\n", 
 	 node->lpos.x, node->lpos.y, node->pos.lat, node->pos.lon);
   
   /* now update the visual representation of the node */
-  GnomeCanvasItem *item = map_item->item;
-  glong x = node->lpos.x, y = node->lpos.y;
+  canvas_item_t *item = map_item->item;
   
   if(!node->ways) 
-    gnome_canvas_item_set(item,
-			  "x1", (double)x-RADIUS, "x2", (double)x+RADIUS,
-			  "y1", (double)y-RADIUS, "y2", (double)y+RADIUS,
-			  NULL);
+    canvas_item_set_pos(item, &node->lpos, RADIUS);
   else {
     /* only move item of it's not the highlight. highlight will be */
     /* redrawed entirely lateron */
     if(!map_item->highlight)
-      gnome_canvas_item_set(item,
-			    "x1", (double)x-S_RADIUS, "x2", (double)x+S_RADIUS,
-			    "y1", (double)y-S_RADIUS, "y2", (double)y+S_RADIUS,
-			    NULL);
+      canvas_item_set_pos(item, &node->lpos, S_RADIUS);
   }
 
   /* update ways, node is part of */
-  way_t *way = gpx->way;
+  way_t *way = osm->way;
   while(way) {
-    if(gpx_node_in_way(way, node)) {
+    if(osm_node_in_way(way, node)) {
       printf("node is part of way #%ld\n", way->id);
       
       /* rebuild ways positions */
       
       /* allocate space for nodes */
-      guint nodes = gpx_way_number_of_nodes(way);
+      guint nodes = osm_way_number_of_nodes(way);
       if(nodes > 1) {
-	GnomeCanvasPoints *points = gnome_canvas_points_new(nodes);
+	canvas_points_t *points = canvas_points_new(nodes);
       
 	int node = 0;
 	node_chain_t *node_chain = way->node_chain;
 	while(node_chain) {
-	  /* mirror map vertically within bounds */
-	  points->coords[node++] = node_chain->node->lpos.x;
-	  points->coords[node++] = node_chain->node->lpos.y;
+	  canvas_point_set_pos(points, node++, &node_chain->node->lpos);
 	  node_chain = node_chain->next;
 	}
 	
 	/* do something with points */
 	map_item_chain_t *chain = way->map_item_chain;
 	while(chain) {
-	  gnome_canvas_item_set(chain->map_item->item, 
-				"points", points, NULL);
+	  canvas_item_set_points(chain->map_item->item, points);
 	  chain = chain->next;
 	}
 	
-	gnome_canvas_points_free(points);
+	canvas_points_free(points);
       }
     }
     
@@ -809,7 +719,7 @@ static void map_node_move(appdata_t *appdata, map_item_t *map_item,
   }
   
   /* and mark the node as dirty */
-  node->flags |= GPX_FLAG_DIRTY;
+  node->flags |= OSM_FLAG_DIRTY;
 
   /* update highlight */
   map_highlight_refresh(appdata);
@@ -853,11 +763,13 @@ static void map_button_press(map_t *map, gint x, gint y) {
   map->pen_down.at.x = x;
   map->pen_down.at.y = y;
   map->pen_down.drag = FALSE;     // don't assume drag yet
-  
+
+#ifdef USE_GNOMECANVAS  
   /* save initial scroll offset */
   gnome_canvas_get_scroll_offsets(GNOME_CANVAS(map->canvas), 
-				  &map->pen_down.so.x, &map->pen_down.so.y);
-  
+		  &map->pen_down.so.x, &map->pen_down.so.y);
+#endif  
+
   /* determine wether this press was on an item */
   map->pen_down.on_item = map_real_item_at(map, x, y);
   
@@ -903,7 +815,7 @@ static void map_button_release(map_t *map, gint x, gint y) {
 	if(!map->selected.item) {
 	  printf("  item has no visible representation to push\n");
 	} else {
-	  gnome_canvas_item_lower_to_bottom(map->selected.item);
+	  canvas_item_to_bottom(map->selected.item);
 	  
 	  /* update clicked item, to correctly handle the click */
 	  map->pen_down.on_item = 
@@ -933,10 +845,9 @@ static void map_button_release(map_t *map, gint x, gint y) {
     printf("released after NODE ADD\n");
     map_hl_cursor_clear(map);
 
-    double px, py;
     /* convert mouse position to canvas (world) position */
-    gnome_canvas_window_to_world(GNOME_CANVAS(map->canvas), x, y, &px, &py);
-    node_t *node = gpx_node_new(map->appdata->gpx, px, py);
+    canvas_window2world(map->canvas, x, y, &x, &y);
+    node_t *node = osm_node_new(map->appdata->osm, x, y);
     map_node_draw(map, node);
 
     map->action = MAP_ACTION_IDLE;
@@ -978,18 +889,15 @@ static void map_touchnode_update(appdata_t *appdata, gint x, gint y) {
   g_assert(map->pen_down.on_item->type == MAP_TYPE_NODE);
 
   /* check if we are close to one of the other nodes */
-  double wx, wy;
-  gnome_canvas_window_to_world(GNOME_CANVAS(appdata->map->canvas), 
-			       x, y, &wx, &wy);
-  gint iwx = wx, iwy = wy;
-  node_t *node = appdata->gpx->node;
+  canvas_window2world(appdata->map->canvas, x, y, &x, &y);
+  node_t *node = appdata->osm->node;
   while(node) {
     /* don't highlight the dragged node itself and don't highlight */
     /* deleted ones */
     if((node != map->pen_down.on_item->node) && 
-       (!(node->flags & GPX_FLAG_DELETED))) {
-      gint nx = iwx - node->lpos.x;
-      gint ny = iwy - node->lpos.y;
+       (!(node->flags & OSM_FLAG_DELETED))) {
+      gint nx = x - node->lpos.x;
+      gint ny = y - node->lpos.y;
 	  
       if(nx*nx + ny*ny < RADIUS*RADIUS)
 	map_hl_touchnode_draw(map, node->lpos.x, node->lpos.y);
@@ -1059,10 +967,8 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
 #else
     case '+':
 #endif
-      appdata->map->zoom *= 2;
+      map_set_zoom(appdata->map, appdata->map->zoom*2);
       printf("zoom is now %f\n", appdata->map->zoom);
-      gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(appdata->map->canvas), 
-				       appdata->map->zoom);
       return TRUE;
       break;
 
@@ -1071,10 +977,8 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
 #else
     case '-':
 #endif
-      appdata->map->zoom /= 2;
+      map_set_zoom(appdata->map, appdata->map->zoom/2);
       printf("zoom is now %f\n", appdata->map->zoom);
-      gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(appdata->map->canvas), 
-				       appdata->map->zoom);
       return TRUE;
       break;
     }
@@ -1094,20 +998,34 @@ GtkWidget *map_new(appdata_t *appdata) {
   map->appdata = appdata;
   map->action = MAP_ACTION_IDLE;
 
+#ifdef USE_GNOMECANVAS
   map->canvas = gnome_canvas_new_aa();  // _aa
 
   /* create the groups */
-  map_group_t group;
-  for(group = 0; group < MAP_GROUPS; group++) {
+  canvas_group_t group;
+  for(group = 0; group < CANVAS_GROUPS; group++) 
     map->group[group] = gnome_canvas_item_new(
 	       	gnome_canvas_root(GNOME_CANVAS(map->canvas)),
 	       	GNOME_TYPE_CANVAS_GROUP,
 	       	NULL);       
-  }
 
   gtk_widget_modify_bg(map->canvas, GTK_STATE_NORMAL, 
 		       &map->canvas->style->white);
 
+#else
+  map->canvas = goo_canvas_new();
+
+  g_object_set(G_OBJECT(map->canvas), "anchor", GTK_ANCHOR_CENTER, NULL);
+
+  GooCanvasItem *root = goo_canvas_get_root_item(GOO_CANVAS(map->canvas));
+
+  /* create the groups */
+  canvas_group_t group;
+  for(group = 0; group < CANVAS_GROUPS; group++) 
+    map->group[group] = goo_canvas_group_new(root, NULL);       
+
+#endif
+
   gtk_widget_set_events(map->canvas,
                           GDK_BUTTON_PRESS_MASK
 			| GDK_BUTTON_RELEASE_MASK
@@ -1127,10 +1045,6 @@ GtkWidget *map_new(appdata_t *appdata) {
   gtk_signal_connect(GTK_OBJECT(map->canvas), 
      "destroy", G_CALLBACK(map_destroy_event), appdata);
 
-  int cx, cy;
-  gnome_canvas_w2c(GNOME_CANVAS(map->canvas), 0,0, &cx, &cy);
-  gnome_canvas_scroll_to(GNOME_CANVAS(map->canvas), cx, cy);
-
   return map->canvas;
 }
 
@@ -1138,17 +1052,15 @@ void map_init(appdata_t *appdata) {
   map_t *map = appdata->map;
 
   /* set initial zoom */
-  gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(map->canvas), 
-				   map->zoom);
+  map_set_zoom(map, map->zoom);
+  map_init_colors(appdata->potlatch, appdata->osm);
+  map_draw(map, appdata->osm);
 
-  map_init_colors(appdata->potlatch, appdata->gpx);
-  map_draw(map, appdata->gpx);
-
-  /* adjust scroll region to map bounds */
-  gnome_canvas_set_scroll_region(GNOME_CANVAS(map->canvas), 
-	 appdata->gpx->bounds->min.x, appdata->gpx->bounds->min.y,
-	 appdata->gpx->bounds->max.x, appdata->gpx->bounds->max.y);
+  canvas_set_bounds(map->canvas,
+	appdata->osm->bounds->min.x, appdata->osm->bounds->min.y,
+	appdata->osm->bounds->max.x, appdata->osm->bounds->max.y);
 
+  canvas_scroll_to(map->canvas, 0,0);
 }
 
 
@@ -1178,25 +1090,31 @@ void map_clear(appdata_t *appdata) {
 
   /* remove a possibly existing highlight */
   map_item_deselect(appdata);
+  
+  canvas_group_t group;
+  for(group=0;group<CANVAS_GROUPS;group++) {
 
-  map_group_t group;
-  for(group=0;group<MAP_GROUPS;group++) {
     /* destroy the entire group */
-    gtk_object_destroy(GTK_OBJECT(map->group[group]));
+    canvas_item_destroy(map->group[group]);
 
+#ifdef USE_GNOMECANVAS
     /* and create an empty new one */
     map->group[group] = gnome_canvas_item_new(
 		      gnome_canvas_root(GNOME_CANVAS(map->canvas)),
 		      GNOME_TYPE_CANVAS_GROUP,
 		      NULL);       
+#else
+    GooCanvasItem *root = goo_canvas_get_root_item(GOO_CANVAS(map->canvas));
+    map->group[group] = goo_canvas_group_new(root, NULL);       
+#endif
   }
 }
 
 void map_paint(appdata_t *appdata) {
   map_t *map = appdata->map;
 
-  map_init_colors(appdata->potlatch, appdata->gpx);
-  map_draw(map, appdata->gpx);
+  map_init_colors(appdata->potlatch, appdata->osm);
+  map_draw(map, appdata->osm);
 }
 
 /* called from several icons like e.g. "node_add" */
@@ -1208,7 +1126,7 @@ void map_action_set(appdata_t *appdata, map_action_t action) {
 static void map_item_chain_destroy(map_item_chain_t *chain) {
   while(chain) {
     map_item_chain_t *next = chain->next;
-    gtk_object_destroy(GTK_OBJECT(chain->map_item->item));
+    canvas_item_destroy(chain->map_item->item);
     chain = next;
   }
 }
@@ -1229,7 +1147,7 @@ void map_delete_selected(appdata_t *appdata) {
 
     /* check if this node is part of a way with two nodes only. */
     /* we cannot delete this as this would also delete the way */
-    way_chain_t *way_chain = gpx_node_to_way(appdata->gpx, item.node);
+    way_chain_t *way_chain = osm_node_to_way(appdata->osm, item.node);
     if(way_chain) {
       gboolean short_way = FALSE;
       
@@ -1237,7 +1155,7 @@ void map_delete_selected(appdata_t *appdata) {
       while(way_chain && !short_way) {
 	way_chain_t *next = way_chain->next;
 
-	if(gpx_way_number_of_nodes(way_chain->way) <= 2) 
+	if(osm_way_number_of_nodes(way_chain->way) <= 2) 
 	  short_way = TRUE;
 
 	g_free(way_chain);
@@ -1253,7 +1171,7 @@ void map_delete_selected(appdata_t *appdata) {
     }
 
     /* check if this node is part of a relation */
-    relation_chain_t *rel_chain = gpx_node_to_relation(appdata->gpx, item.node);
+    relation_chain_t *rel_chain = osm_node_to_relation(appdata->osm, item.node);
     if(rel_chain) {
       
       /* free the chain of relations */
@@ -1264,7 +1182,7 @@ void map_delete_selected(appdata_t *appdata) {
       }
 
       messagef(GTK_WIDGET(appdata->window), _("Node in relation"), 
-	       _("This node cannot be deleted (yet) as it's part of a relation"));
+       _("This node cannot be deleted (yet) as it's part of a relation"));
 
       return;
     }
@@ -1273,13 +1191,13 @@ void map_delete_selected(appdata_t *appdata) {
     map_item_chain_destroy(item.node->map_item_chain);
 
     /* and mark it "deleted" in the database */
-    way_chain_t *chain = gpx_node_delete(appdata->gpx, item.node, FALSE);
+    way_chain_t *chain = osm_node_delete(appdata->osm, item.node, FALSE);
 
     /* redraw all affected ways */
     while(chain) {
       way_chain_t *next = chain->next;
 
-      if(gpx_way_number_of_nodes(chain->way) == 1) {
+      if(osm_way_number_of_nodes(chain->way) == 1) {
 	printf("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n");
 	printf("TODO: Only 1 node left on way -> delete way also\n");
 	printf("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX\n");
@@ -1308,3 +1226,105 @@ void map_delete_selected(appdata_t *appdata) {
     break;
   }
 }
+
+/* ----------------------- track related stuff ----------------------- */
+
+void map_track_draw_seg(map_t *map, track_seg_t *seg) {
+  /* a track_seg needs at least 2 points to be drawn */
+  guint pnum = track_seg_points(seg);
+  printf("seg of length %d\n", pnum);
+
+  if(pnum == 1) {
+    g_assert(!seg->item);
+
+    seg->item = canvas_circle_new(map, CANVAS_GROUP_TRACK,
+		  seg->track_point->lpos.x, seg->track_point->lpos.y, 
+		  1.5*WAY_WIDTH, 0, TRACK_COLOR, NO_COLOR);
+  }
+
+  if(pnum > 1) {
+    
+    /* allocate space for nodes */
+    canvas_points_t *points = canvas_points_new(pnum);
+    
+    int point = 0;
+    track_point_t *track_point = seg->track_point;
+    while(track_point) {
+      points->coords[point++] = track_point->lpos.x;
+      points->coords[point++] = track_point->lpos.y;
+      track_point = track_point->next;
+    }
+    
+    /* there may be a circle (one point line) */
+    if(seg->item)
+      canvas_item_destroy(seg->item);
+
+    seg->item = canvas_polyline_new(map, CANVAS_GROUP_TRACK,
+		  points, 3*WAY_WIDTH, TRACK_COLOR);
+
+    canvas_points_free(points);
+  }
+}
+
+void map_track_update_seg(map_t *map, track_seg_t *seg) {
+  /* a track_seg needs at least 2 points to be drawn */
+  guint pnum = track_seg_points(seg);
+  printf("seg of length %d\n", pnum);
+
+  if(pnum > 1) {
+    
+    /* allocate space for nodes */
+    canvas_points_t *points = canvas_points_new(pnum);
+    
+    int point = 0;
+    track_point_t *track_point = seg->track_point;
+    while(track_point) {
+      canvas_point_set_pos(points, point++, &track_point->lpos);
+      track_point = track_point->next;
+    }
+    
+    g_assert(seg->item);
+    canvas_item_set_points(seg->item, points);
+    canvas_points_free(points);
+  }
+}
+
+void map_track_draw(map_t *map, track_t *track) {
+  track_seg_t *seg = track->track_seg;
+
+  /* draw all segments */
+  while(seg) {
+    map_track_draw_seg(map, seg);
+    seg = seg->next;
+  }
+}
+
+void map_track_remove(appdata_t *appdata) {
+  track_t *track = appdata->track.track;
+
+  printf("removing track\n");
+
+  g_assert(track);
+
+  /* remove all segments */
+  track_seg_t *seg = track->track_seg;
+  while(seg) {
+    if(seg->item) {
+      canvas_item_destroy(seg->item);
+      seg->item = NULL;
+    }
+
+    seg = seg->next;
+  }
+}
+
+void map_track_pos(appdata_t *appdata, lpos_t *lpos) {
+  if(appdata->track.gps_item) {
+    canvas_item_destroy(appdata->track.gps_item);
+    appdata->track.gps_item = NULL;
+  }
+
+  if(lpos)
+    appdata->track.gps_item = canvas_circle_new(appdata->map, CANVAS_GROUP_GPS, 
+	lpos->x, lpos->y, 1.5 * WAY_WIDTH, 0, TRACK_GPS_COLOR, NO_COLOR);
+}
